package dev.fyloz.colorrecipesexplorer.dtos

import com.fasterxml.jackson.annotation.JsonIgnore
import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.model.account.Group
import java.time.LocalDate
import javax.validation.constraints.Max
import javax.validation.constraints.Min
import javax.validation.constraints.NotBlank
import javax.validation.constraints.Pattern

data class RecipeDto(
    override val id: Long = 0L,

    val name: String,

    val description: String,

    val color: String,

    val gloss: Byte,

    val sample: Int?,

    val approbationDate: LocalDate?,

    val approbationExpired: Boolean,

    val remark: String,

    val company: CompanyDto,

    val mixes: List<MixDto>,

    val groupsInformation: List<RecipeGroupInformationDto>
) : EntityDto {
    val mixTypes: Collection<MixTypeDto>
        @JsonIgnore
        get() = mixes.map { it.mixType }
}

data class RecipeSaveDto(
    @field:NotBlank
    val name: String,

    @field:NotBlank
    val description: String,

    @field:NotBlank
    @field:Pattern(regexp = Constants.ValidationRegexes.VALIDATION_COLOR_PATTERN)
    val color: String,

    @field:Min(0, message = Constants.ValidationMessages.RANGE_OUTSIDE_PERCENTS)
    @field:Max(100, message = Constants.ValidationMessages.RANGE_OUTSIDE_PERCENTS)
    val gloss: Byte,

    @field:Min(0, message = Constants.ValidationMessages.SIZE_GREATER_OR_EQUALS_ZERO)
    val sample: Int?,

    val approbationDate: LocalDate?,

    val remark: String?,

    val companyId: Long
)

data class RecipeUpdateDto(
    val id: Long,

    @field:NotBlank
    val name: String,

    @field:NotBlank
    val description: String,

    @field:NotBlank
    @field:Pattern(regexp = Constants.ValidationRegexes.VALIDATION_COLOR_PATTERN)
    val color: String,

    @field:Min(0, message = Constants.ValidationMessages.RANGE_OUTSIDE_PERCENTS)
    @field:Max(100, message = Constants.ValidationMessages.RANGE_OUTSIDE_PERCENTS)
    val gloss: Byte,

    @field:Min(0, message = Constants.ValidationMessages.SIZE_GREATER_OR_EQUALS_ZERO)
    val sample: Int?,

    val approbationDate: LocalDate?,

    val remark: String?,

    val steps: List<RecipeGroupStepsDto>
)

data class RecipeGroupInformationDto(
    override val id: Long = 0L,

    val group: Group,

    val note: String? = null,

    val steps: List<RecipeStepDto> = listOf()
) : EntityDto

data class RecipeGroupStepsDto(
    val groupId: Long,

    val steps: List<RecipeStepDto>
)

data class RecipeGroupNoteDto(
    val groupId: Long,

    val content: String?
)

data class RecipePublicDataDto(
    val recipeId: Long,

    val notes: List<RecipeGroupNoteDto>,

    val mixesLocation: List<MixLocationDto>
)