package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.MaterialType
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository

@Repository
interface MaterialTypeRepository : JpaRepository<MaterialType, Long> {
    /** Checks if a system material type with the given [id] exists. */
    fun existsByIdAndSystemTypeIs(id: Long, systemType: Boolean): Boolean

    /** Checks if a material type with the given [name] and a different [id] exists. */
    fun existsByNameAndIdNot(name: String, id: Long): Boolean

    /** Checks if a material type with the given [prefix] and a different [id] exists. */
    fun existsByPrefixAndIdNot(prefix: String, id: Long): Boolean

    /** Find all material types which are or not [systemType]s. */
    fun findAllBySystemTypeIs(systemType: Boolean): Collection<MaterialType>

    /** Find the material type with the given [name]. */
    fun findByName(name: String): MaterialType?

    /** Checks if a material depends on the material type with the given [id]. */
    @Query(
        """
        select case when(count(m) > 0) then true else false end
        from Material m where m.materialType.id = :id
        """
    )
    fun isUsedByMaterial(id: Long): Boolean
}
