package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewCatalog
import dev.fyloz.colorrecipesexplorer.dtos.MaterialDto
import dev.fyloz.colorrecipesexplorer.dtos.MaterialSaveDto
import dev.fyloz.colorrecipesexplorer.logic.MaterialLogic
import org.springframework.context.annotation.Profile
import org.springframework.http.MediaType
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import javax.validation.Valid

@RestController
@RequestMapping(Constants.ControllerPaths.MATERIAL)
@Profile("!emergency")
@PreAuthorizeViewCatalog
class MaterialController(
    private val materialLogic: MaterialLogic
) {
    @GetMapping
    fun getAll() =
        ok(materialLogic.getAll())

    @GetMapping("notmixtype")
    fun getAllNotMixType() =
        ok(materialLogic.getAllNotMixType())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(materialLogic.getById(id))

    @PostMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAuthority('EDIT_MATERIALS')")
    fun save(@Valid material: MaterialSaveDto, simdutFile: MultipartFile?) =
        created<MaterialDto>(Constants.ControllerPaths.MATERIAL) {
            materialLogic.save(material.copy(simdutFile = simdutFile))
        }

    @PutMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAuthority('EDIT_MATERIALS')")
    fun update(@Valid material: MaterialSaveDto, simdutFile: MultipartFile?) =
        noContent {
            materialLogic.update(material.copy(simdutFile = simdutFile))
        }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_MATERIALS')")
    fun deleteById(@PathVariable id: Long) =
        noContent {
            materialLogic.deleteById(id)
        }

    @GetMapping("mix/create/{recipeId}")
    fun getAllForMixCreation(@PathVariable recipeId: Long) =
        ok(materialLogic.getAllForMixCreation(recipeId))

    @GetMapping("mix/update/{mixId}")
    fun getAllForMixUpdate(@PathVariable mixId: Long) =
        ok(materialLogic.getAllForMixUpdate(mixId))
}

