package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.config.annotations.ServiceComponent
import dev.fyloz.colorrecipesexplorer.dtos.MixDto
import dev.fyloz.colorrecipesexplorer.model.Mix
import dev.fyloz.colorrecipesexplorer.repository.MixRepository

interface MixService : Service<MixDto, Mix, MixRepository> {
    /** Gets all mixes with the mix type with the given [mixTypeId]. */
    fun getAllByMixTypeId(mixTypeId: Long): Collection<MixDto>

    /** Updates the [location] of the mix with the given [id]. */
    fun updateLocationById(id: Long, location: String?)
}

@ServiceComponent
class DefaultMixService(
    repository: MixRepository,
    private val mixTypeService: MixTypeService,
    private val mixMaterialService: MixMaterialService
) : BaseService<MixDto, Mix, MixRepository>(repository), MixService {
    override fun getAllByMixTypeId(mixTypeId: Long) = repository.findAllByMixTypeId(mixTypeId).map(::toDto)
    override fun updateLocationById(id: Long, location: String?) = repository.updateLocationById(id, location)

    override fun toDto(entity: Mix) =
        MixDto(
            entity.id!!,
            entity.location,
            entity.recipeId,
            mixTypeService.toDto(entity.mixType),
            entity.mixMaterials.map(mixMaterialService::toDto)
        )

    override fun toEntity(dto: MixDto) =
        Mix(
            dto.id,
            dto.location,
            dto.recipeId,
            mixTypeService.toEntity(dto.mixType),
            dto.mixMaterials.map(mixMaterialService::toEntity)
        )
}