package dev.fyloz.colorrecipesexplorer.logic

import dev.fyloz.colorrecipesexplorer.dtos.RecipeGroupInformationDto
import dev.fyloz.colorrecipesexplorer.dtos.RecipeStepDto
import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionError
import dev.fyloz.colorrecipesexplorer.exception.InvalidPositionsException
import dev.fyloz.colorrecipesexplorer.model.account.Group
import dev.fyloz.colorrecipesexplorer.service.RecipeStepService
import dev.fyloz.colorrecipesexplorer.utils.PositionUtils
import io.mockk.*
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows

class DefaultRecipeStepLogicTest {
    private val recipeStepServiceMock = mockk<RecipeStepService>()

    private val recipeStepLogic = DefaultRecipeStepLogic(recipeStepServiceMock)

    @AfterEach
    internal fun afterEach() {
        clearAllMocks()
    }

    @Test
    fun validateGroupInformationSteps_normalBehavior_callsValidateSteps() {
        // Arrange
        mockkObject(PositionUtils)
        every { PositionUtils.validate(any()) } just runs

        val group = Group(1L, "Unit test group")
        val steps = listOf(RecipeStepDto(1L, 1, "A message"))
        val groupInfo = RecipeGroupInformationDto(1L, group, "A note", steps)

        // Act
        recipeStepLogic.validateGroupInformationSteps(groupInfo)

        // Assert
        verify {
            PositionUtils.validate(steps.map { it.position })
        }
    }

    @Test
    fun validateGroupInformationSteps_invalidSteps_throwsInvalidGroupStepsPositionsException() {
        // Arrange
        val errors = setOf(InvalidPositionError("error", "An unit test error"))

        mockkObject(PositionUtils)
        every { PositionUtils.validate(any()) } throws InvalidPositionsException(errors)

        val group = Group(1L, "Unit test group")
        val steps = listOf(RecipeStepDto(1L, 1, "A message"))
        val groupInfo = RecipeGroupInformationDto(1L, group, "A note", steps)

        // Act
        // Assert
        assertThrows<InvalidGroupStepsPositionsException> { recipeStepLogic.validateGroupInformationSteps(groupInfo) }
    }
}